import { } from '@stencil/core';
var FrostedGlass = /** @class */ (function () {
    function FrostedGlass() {
        this.blurAmount = '5px';
        this.directions = ['top', 'left', 'right'];
        this.topOffset = 0;
        this.ticking = {};
    }
    FrostedGlass.prototype.updateBackground = function () {
        this.requestTick('backgroundUpdate');
    };
    FrostedGlass.prototype.backgroundChangeHandler = function () {
        this.backgroundUpdate();
    };
    FrostedGlass.prototype.blurAmountChangeHandler = function (newValue) {
        this.updateFilter(newValue);
    };
    FrostedGlass.prototype.componentDidLoad = function () {
        this.setBlurId();
        this.createNewElements();
        this.addBaseStyles();
        this.initListeners();
    };
    FrostedGlass.prototype.componentDidUnload = function () {
        this.removeElements();
        this.removeListeners();
    };
    FrostedGlass.prototype.onResize = function () {
        this.requestTick('resizeUpdate');
    };
    FrostedGlass.prototype.onScroll = function () {
        this.latestKnownScrollY = window.scrollY;
        this.requestTick('scrollUpdate');
    };
    FrostedGlass.prototype.render = function () {
        return (h("slot", null));
    };
    FrostedGlass.prototype.createNewElements = function () {
        this.blurContainer = document.createElement('div');
        this.blurContainer.setAttribute('id', "blur-container-" + this.blurId);
        this.blurContent = document.createElement('div');
        this.blurContainer.appendChild(this.blurContent);
        document.querySelector('body').appendChild(this.blurContainer);
        this.requestTick('backgroundUpdate');
    };
    FrostedGlass.prototype.removeElements = function () {
        this.blurContainer.remove();
    };
    FrostedGlass.prototype.removeListeners = function () {
        window.removeEventListener('scroll', this.onScroll);
        window.removeEventListener('resize', this.onResize);
    };
    FrostedGlass.prototype.addBaseStyles = function () {
        Object.assign(this.blurContainer.style, {
            overflow: 'hidden',
            transform: 'translate3d(0, 0, 0)'
        });
        this.blurContent.style.position = 'absolute';
        this.updateFilter(this.blurAmount);
    };
    FrostedGlass.prototype.updateFilter = function (blurAmount) {
        this.blurContent.style.filter = "blur(" + blurAmount + ")";
    };
    FrostedGlass.prototype.initListeners = function () {
        if (window.getComputedStyle(this.el).position === 'fixed') {
            this.onScroll = this.onScroll.bind(this);
            window.addEventListener('scroll', this.onScroll);
        }
        this.onResize = this.onResize.bind(this);
        window.addEventListener('resize', this.onResize);
        this.onResize();
        this.onScroll();
    };
    // @ts-ignore
    FrostedGlass.prototype.resizeUpdate = function () {
        var _this = this;
        var elementStyle = window.getComputedStyle(this.el);
        var appStyle = window.getComputedStyle(this.background);
        this.topOffset = parseInt(elementStyle.top, 10);
        ['position', 'height'].concat(this.directions).forEach(function (item) {
            _this.blurContainer.style[item] = elementStyle[item];
        });
        this.directions.forEach(function (item) { _this.blurContent.style[item] = "-" + elementStyle[item]; });
        this.blurContent.style.width = appStyle.width;
        this.scrollUpdate();
        this.ticking.resizeUpdate = false;
    };
    FrostedGlass.prototype.scrollUpdate = function () {
        var scrollOffset = this.latestKnownScrollY + this.topOffset;
        this.blurContent.style.top = "-" + scrollOffset + "px";
        this.ticking.scrollUpdate = false;
    };
    FrostedGlass.prototype.requestTick = function (functionName) {
        if (!this.ticking[functionName]) {
            requestAnimationFrame(this[functionName].bind(this));
        }
        this.ticking[functionName] = true;
    };
    // @ts-ignore
    FrostedGlass.prototype.backgroundUpdate = function () {
        if (!this.backgroundSelector) {
            return;
        }
        this.background = document.querySelector(this.backgroundSelector);
        var backgroundClone = document.importNode(this.background, true);
        var selfClone = backgroundClone.querySelector("[data-blur-id=\"" + this.blurId + "\"]");
        if (selfClone) {
            selfClone.remove();
        }
        this.blurContent.innerHTML = '';
        this.blurContent.appendChild(backgroundClone);
        this.ticking.backgroundUpdate = false;
    };
    FrostedGlass.prototype.setBlurId = function () {
        this.blurId = "" + Math.random();
        this.el.dataset.blurId = this.blurId;
    };
    return FrostedGlass;
}());
export { FrostedGlass };
